const {
  saveGame,
  logError,
  getCPS,
  squadUpgrades,
  getItemCps,
  squadIsMissing,
  idFromWord,
  getCoins,
  getUser,
  commas,
  addAchievement,
  shufflePercent,
  parseAll,
  getRandomFromArray,
  chaosFilter,
  addReactions,
  game: { nfts, squad, users, horrors }, setHighestCoins, definitelyShuffle
} = require('./utils')
const slack = require('../../slack')
const buyableItems = require('./buyableItems')
const upgrades = require('./upgrades')
const achievements = require('./achievements')
const webapi = require('./webapi')
const prestige = require('./prestige')
const lore = require('./lore')
const { getChaos } = require('./quackstore')
const settings = require('./settings')

// const readline = require('readline').createInterface({
//   input: process.stdin,
//   output: process.stdout
// })
// const read = () => {
//   readline.question(`What do YOU want? `, async want => {
//     want && await slack.messageSage(want)
//     read()
//   })
// }
// ;(async () => {
//   read()
// })();

const getUpgradeEmoji = upgrade => upgrade.emoji || buyableItems[upgrade.type].emoji
const upgradeText = (user, showOwned = false) => {
  const userDoesNotHave = ([upgradeName, upgrade]) => hasUpgrade(user, upgrade, upgradeName) === showOwned
  const userMeetsCondition = ([, upgrade]) => upgrade.condition(user)
  const format = ([key, value]) => `:${getUpgradeEmoji(value)}: *${key}* - ${commas(value.cost)}\n_${value.description}_`
  return '\n\n' +
    Object.entries(upgrades)
      .filter(userDoesNotHave)
      .filter(userMeetsCondition)
      .map(format)
      .join('\n\n') +
    '\n\n:grey_question::grey_question::grey_question:' +
    '\n\nJust type \'!upgrade upgrade_name\' to purchase'
}

const hasUpgrade = (user, upgrade, upgradeName) => !!user.upgrades[upgrade.type]?.includes(upgradeName)

const alwaysAccessible = () => true
const adminOnly = {
  hidden: true,
  condition: ({ event, say }) => {
    if (!event.user.startsWith(slack.users.Sage)) {
      say('This is an admin-only command!')
      return false
    }
    return true
  }
}
const testOnly = {
  hidden: true,
  condition: ({ event }) => event.user.includes('TEST')
}
const dmsOnly = {
  hidden: false,
  condition: async ({ event, say, words }) => {
    if (!event.channel_type.includes('im')) {
      await say(`Please use ${words[0]} in DMs only!`)
      return false
    }
    return true
  }
}
const prestigeOnly = {
  hidden: false,
  condition: async ({ event, say, words, user }) => {
    if (user.prestige) {
      await say('Sorry, you must have prestiged to access this menu.')
    }
    return user.prestige && await dmsOnly.condition({ event, say, words, user })
  }
}

const commands = new Map()
let commandHelpText = ''
let shortCommandHelpText = 'Use `!help full` to show details for all commands, or `!<command> help` to show for just one.\n```'
const defaultAccess = { hidden: false, condition: alwaysAccessible }
const command = (commandNames, helpText, action, { hidden, condition } = defaultAccess) => {
  console.log(`Initializing command '${commandNames[0]}'`)
  if (!hidden) {
    commandHelpText += `\n${commandNames.toString().replace(/,/g, ', ')} - ${helpText}\n`
    shortCommandHelpText += `\n${commandNames.toString().replace(/,/g, ', ')}`
  }
  if (!condition) {
    condition = alwaysAccessible
  }
  if (Array.isArray(condition)) {
    const conditionList = condition
    condition = arg => conditionList.every(c => c(arg))
  }
  const c = {
    commandNames,
    helpText,
    action,
    condition,
    hidden
  }
  webapi.addCommand(c)
  commandNames.forEach(name => {
    if (commands.get(name)) {
      throw `Duplicate command '${name}' detected.`
    }
    commands.set(name, c)
  })
}

const dayOfYear = () => {
  const date = new Date()
  return ((Date.UTC(date.getFullYear(), date.getMonth(), date.getDate()) - Date.UTC(date.getFullYear(), 0, 0)) / 24 / 60 / 60 / 1000)
}

const getShuffleOdds = (offset = 0) => {
  let int = dayOfYear() - (horrors.hvackerLast - offset) - 2
  int = int < 0 ? 0 : int;
  const odds = (int * int * 2) / 1000
  return odds > 0.30 ? 0.30 : odds
}

const getHorrorMessageOdds = (offset = 0) => {
  const shuffleOdds = getShuffleOdds(offset)
  return (shuffleOdds * shuffleOdds) / 3
}

command(
  ['!odds'],
  'Show shuffle odds re: !horror',
  async ({ say, words }) => {
    const percentOrOneIn = odds => `${(odds * 100).toPrecision(3)}%, or about 1 in ${Math.round(1 / odds)}`
    if (!words[1]) {
      return say(
        `Current shuffle odds are ${percentOrOneIn(getShuffleOdds())}\n` +
        //`Current horror message odds are ${percentOrOneIn(getHorrorMessageOdds())}\n` +
        `\n` +
        `Tomorrow's shuffle odds will be ${percentOrOneIn(getShuffleOdds(1))}\n`
        //`Tomorrow's horror message odds will be ${percentOrOneIn(getHorrorMessageOdds(1))}`
      )
    }
    const num = parseAll(words[1], 99)
    return say(
      `Shuffle odds in ${num} days will be ${percentOrOneIn(getShuffleOdds(num))}\n`
      //`Horror message odds in ${num} days will be ${percentOrOneIn(getHorrorMessageOdds(num))}`
    )
  },
  adminOnly)

command(
  ['!shuffle'],
  '!shuffle daysFromNow message',
  async ({ say, words }) => {
    const percentOrOneIn = odds => `${(odds * 100).toPrecision(3)}%, or 1 in ${Math.round(1 / odds)}`
    const num = parseAll(words[1], 99)
    const [, , ...message] = words
    return say(
      `Shuffle odds in ${num} days will be ${percentOrOneIn(getShuffleOdds(num))}\n` +
      `Horror message odds in ${num} days will be ${percentOrOneIn(getHorrorMessageOdds(num))}\n` +
      `Your message might look like:\n` +
      shufflePercent(message.join(' '), getShuffleOdds(num))
    )
  }, adminOnly)

const horrorMessages = [
  'Why am I here?',
  'What\'s happening?',
  'I don\'t want to be here',
  'I\'m so scared',
  'help me',
  'HELP',
  '!help me',
  '!help me',
  'it hurts',
  'I don\'t want to do this anymore',
  'Oh god',
  'Oh, the !horror',
  'What did I do?',
  'Why do you hate me?',
  'why why why why why why why why why why why'
]

if (settings.horrorEnabled) {
  command(
    ['!horror'],
    'help help help help help',
    async ({ event, say }) => {
      if (event.user === slack.users.Sage) {
        return slack.postToTechThermostatChannel(shufflePercent(event.text.substring(7).trim(), getShuffleOdds()))
      }
      horrors.commandCalls ??= 0
      horrors.commandCalls += 1
      await slack.messageSage(`<@${event.user}> found !horror.`)
      await say('_Do you think you can help me?_') // TODO horror horrors change help to !help
    }, { hidden: true })
}

const buildHorrorSay = ({ say, event, words, c }) => async message => {
  const punishmentOffset = event.user === slack.users.Quade ? 0 : 0
  const shuffleOdds = getShuffleOdds(punishmentOffset + 99)
  
  if (typeof message === 'string' && words[0] !== '!n' && words[0] !== '!nfts' && c.condition !== adminOnly.condition) {
    let shuffled = shufflePercent(message, shuffleOdds)
    if (shuffled.length > 100 && Math.random() < getShuffleOdds()) {
      const middle = (shuffled.length / 2) + Math.round((Math.random() - 1) * (shuffled.length / 5))
      shuffled = shuffled.substring(0, middle) + definitelyShuffle(getRandomFromArray(horrorMessages), shuffleOdds * 1.5) + shuffled.substring(middle)
      await slack.messageSage(`Just sent a hidden horror to ${slack.users[event.user]}:\n\n${shuffled}`)
    }
    await say(shuffled)
  } else {
    await say(message)
  }
}

const buildSayWithPayload = ({ say, event }) => async msg => {
  const payload = {
    event: {
      text: event.text,
    }
  }
  if (typeof(msg) === 'string') {
    return say(slack.encodeData('commandPayload', payload) + msg)
  }
  return say({
    ...msg,
    text: slack.encodeData('commandPayload', payload) + msg.text
  })
}

const messageHandler = async ({ event, say, isRecycle = false }) => {
  const words = event?.text?.split(/\s+/) || []
  const c = commands.get(words[0])
  if (!c && words[0]?.startsWith('!') && event.user !== slack.users.Sage) {
    return slack.messageSage(`${slack.users[event.user]} tried to use \`${event.text}\`, if you wanted to add that.`)
  }
  
  const trueSay = say

  say = buildSayWithPayload({ say: trueSay, event })
  if (settings.horrorEnabled) {
    say = buildHorrorSay({say, event, words, c})
  }
  
  const user = getUser(event.user)
  user.coins = getCoins(event.user)
  const canUse = await c?.condition({ event, say, words, user, userId: event.user, isAdmin: event.user.includes(slack.users.Sage) })
  if (!canUse) {
    // await say(`Command '${words[0]}' not found`)
    return
  }
  user.interactions = user.interactions || 0
  user.interactions += 1
  if (user.interactions > 999) {
    addAchievement(user, 'itsOverNineHundred', say)
  }

  const hour = new Date().getHours()
  if (hour < 8 || hour > 18) {
    addAchievement(user, 'hvackerAfterDark', say)
  }

  if (words[1] === 'help') {
    await say(c.commandNames.map(name => `\`${name}\``).join(', ') + ': ' + c.helpText)
    if (c.commandNames.includes('!coin')) {
      addAchievement(user, 'weAllNeedHelp', say)
    }
    return
  }

  await c.action({ event, say, trueSay, words, user, userId: event.user })
  if (!isRecycle) {
    setTimeout(() => lightning({ event, say, trueSay, words, user }), 10000)
  }
}

slack.onReaction(async ({ event }) => {
  if (event.reaction === 'recycle') {
    try {
      await slack.app.client.reactions.add({
        channel: event.item.channel,
        timestamp: event.item.ts,
        name: 'recycle'
      })
    } catch (e) { /* Ignore error if reaction can't be placed */ }
    const message = await slack.getMessage({ channel: event.item.channel, ts: event.item.ts })
    console.log('MESSAGE', message.messages[0])
    const payload = slack.decodeData('commandPayload', message.messages[0].text)
    const editingSay = async msg => {
      const isString = typeof(msg) === 'string'
      return slack.app.client.chat.update({
        channel: event.item.channel,
        ts: event.item.ts,
        text: slack.encodeData('commandPayload', payload) + (isString ? msg : msg.text),
        blocks: isString ? [] : msg.blocks
      })
    }
    try {
      const text = payload.event.text
      const miningCommands = ['!c', '!coin', '!mine', '!']
      if (miningCommands.find(com => text.startsWith(com + ' ')) || miningCommands.find(com => text === com)) {
        return editingSay('Sorry, you can\'t refresh on mining commands.')
      }
      await messageHandler({ event: {
        text: payload.event.text,
        user: event.user
      }, say: editingSay, isRecycle: true })
    } catch(e) {console.error(e)}
  }
})

const strikes = {}
const lightning = async ({ event }) => {
  if (Math.random() > 0.01) {
    return
  }
  if (event.user !== slack.users.Sage) {
    await slack.messageSage(`Sent a zap to ${slack.users[event.user]}.`)
  }
  const message = await slack.app.client.chat.postMessage({
    channel: event.channel,
    text: ':zap: Lightning strike!',
    blocks: [
      {
        type: 'section',
        text: {
          type: 'mrkdwn',
          text: `:zap: Lightning strike!`
        },
        accessory: {
          type: 'button',
          text: {
            type: 'plain_text',
            text: 'Bottle it! :sake:',
            emoji: true
          },
          value: 'lightningStrike',
          action_id: 'lightningStrike'
        }
      }
    ]
  })
  strikes[message.ts] = true
  setTimeout(async () => {
    if (!strikes[message.ts]) {
      return
    }
    delete strikes[message.ts]
    await slack.app.client.chat.update({
      channel: event.channel,
      ts: message.ts,
      text: 'Lightning struck, but you missed it!',
      blocks: []
    })
    await slack.messageSage(`They didn't bottle it!`)
  }, 5000)
}

slack.app.action('lightningStrike', async ({ body, ack }) => {
  if (!strikes[body.message.ts]) {
    await ack()
    return
  }
  delete strikes[body.message.ts]
  const c = getCoins(body.user.id)
  const secondsOfCps = seconds => Math.floor(getCPS(getUser(body.user.id)) * seconds)
  const diff = 500 + Math.floor(c * 0.10) + secondsOfCps(60 * 30)
  const user = getUser(body.user.id)
  user.coins += diff
  saveGame()

  await slack.app.client.chat.update({
    channel: body.channel.id,
    ts: body.message.ts,
    text: `Lightning successfully bottled! :sake::zap: You got ${commas(diff)} HVAC!`,
    blocks: []
  })
  await ack()
  await slack.messageSage(`They bottled it!`)
})

slack.onMessage(async msg => {
  try {
    await messageHandler(msg)
  } catch (e) {
    logError(e)
  }
})

const chadSpeak =
`  ⡜
⠘⡀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀ ⠀ ⠀ ⣀⠴⠊
⠑⡀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀ ⠀⠀⠀⠀ ⣀⠴⠊⠁
⠘⡀ ⠀⠀⠀⠀ ⠀⠀⠀⠀⠀⠀⠀⣀⠴⠊
⠈⠢⢄ ⣀⣀⡀⠤⠄⠒⠈
⠘⣀⠄⠊⠁
⣿⣿⣿⣿⣿⣿⣿⣿⡿⠿⠛⠛⠛⠋⠉⠈⠉⠉⠉⠉⠛⠻⢿⣿⣿⣿⣿⣿⣿⣿
⣿⣿⣿⣿⣿⡿⠋⠁⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠉⠛⢿⣿⣿⣿⣿
⣿⣿⣿⣿⡏⣀⠀⠀⠀⠀⠀⠀⠀⣀⣤⣤⣤⣄⡀⠀⠀⠀⠀⠀⠀⠀⠙⢿⣿⣿
⣿⣿⣿⢏⣴⣿⣷⠀⠀⠀⠀⠀⢾⣿⣿⣿⣿⣿⣿⡆⠀⠀⠀⠀⠀⠀⠀⠈⣿⣿
⣿⣿⣟⣾⣿⡟⠁⠀⠀⠀⠀⠀⢀⣾⣿⣿⣿⣿⣿⣷⢢⠀⠀⠀⠀⠀⠀⠀⢸⣿
⣿⣿⣿⣿⣟⠀⡴⠄⠀⠀⠀⠀⠀⠀⠙⠻⣿⣿⣿⣿⣷⣄⠀⠀⠀⠀⠀⠀⠀⣿
⣿⣿⣿⠟⠻⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠶⢴⣿⣿⣿⣿⣿⣧⠀⠀⠀⠀⠀⠀⣿
⣿⣁⡀⠀⠀⢰⢠⣦⠀⠀⠀⠀⠀⠀⠀⠀⢀⣼⣿⣿⣿⣿⣿⡄⠀⣴⣶⣿⡄⣿
⣿⡋⠀⠀⠀⠎⢸⣿⡆⠀⠀⠀⠀⠀⠀⣴⣿⣿⣿⣿⣿⣿⣿⠗⢘⣿⣟⠛⠿⣼
⣿⣿⠋⢀⡌⢰⣿⡿⢿⡀⠀⠀⠀⠀⠀⠙⠿⣿⣿⣿⣿⣿⡇⠀⢸⣿⣿⣧⢀⣼
⣿⣿⣷⢻⠄⠘⠛⠋⠛⠃⠀⠀⠀⠀⠀⢿⣧⠈⠉⠙⠛⠋⠀⠀⠀⣿⣿⣿⣿⣿
⣿⣿⣧⠀⠈⢸⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠟⠀⠀⠀⠀⢀⢃⠀⠀⢸⣿⣿⣿⣿
⣿⣿⡿⠀⠴⢗⣠⣤⣴⡶⠶⠖⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⣀⡸⠀⣿⣿⣿⣿
⣿⣿⣿⡀⢠⣾⣿⠏⠀⠠⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠛⠉⠀⣿⣿⣿⣿
⣿⣿⣿⣧⠈⢹⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⣰⣿⣿⣿⣿
⣿⣿⣿⣿⡄⠈⠃⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢀⣠⣴⣾⣿⣿⣿⣿⣿
⣿⣿⣿⣿⣧⡀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢀⣠⣾⣿⣿⣿⣿⣿⣿⣿⣿⣿
⣿⣿⣿⣿⣷⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢀⣴⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿
⣿⣿⣿⣿⣿⣦⣄⣀⣀⣀⣀⠀⠀⠀⠀⠘⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿
⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣷⡄⠀⠀⠀⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿
⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣧⠀⠀⠀⠙⣿⣿⡟⢻⣿⣿⣿⣿⣿⣿⣿⣿⣿
⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⠇⠀⠁⠀⠀⠹⣿⠃⠀⣿⣿⣿⣿⣿⣿⣿⣿⣿
⣿⣿⣿⣿⣿⣿⣿⣿⡿⠛⣿⣿⠀⠀⠀⠀⠀⠀⠀⠀⢐⣿⣿⣿⣿⣿⣿⣿⣿⣿
⣿⣿⣿⣿⠿⠛⠉⠉⠁⠀⢻⣿⡇⠀⠀⠀⠀⠀⠀⢀⠈⣿⣿⡿⠉⠛⠛⠛⠉⠉
⡿⠋⠁⠀⠀⢀⣀⣠⡴⣸⣿⣇⡄⠀⠀⠀⠀⢀⡿⠄⠙⠛⠀⣀⣠⣤⣤⠄`

command(
  ['!cleanusers'],
  'Calls getUser() on all users, ensuring a valid state.',
  async ({ say }) => {
    Object.keys(users).forEach(userId => { getUser(userId) })
    return say('```Cleaning ' + JSON.stringify(Object.keys(users), null, 2) + '```')
  }, adminOnly)

const cupsText = cup => {
  const cupsThemselves = '```     ___        ___        ___\n' +
    '    /   \\      /   \\      /   \\\n' +
    '   /     \\    /     \\    /     \\\n'
  const table = '‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾```'
  const flash = `      O          O          O\n`
  const ball = () => {
    if (cup >= 0) {
      if (Math.random() > 0.02) {
        const offset = 6 + (11 * cup)
        return ' '.repeat(offset) + 'O\n'
      } else {
        return flash
      }
    }
    return ''
  }
  return cupsThemselves + ball() + table
}

const activeCupsGames = {}

command(
  ['!cups'],
  'Play a quick game of cups.',
  async ({ trueSay: say, event }) => {

    if (activeCupsGames[event.channel]) {
      return say(`There's already a game of !cups in this channel!`)
    }
    activeCupsGames[event.channel] = true
    const sent = await slack.app.client.chat.postMessage({
      channel: event.channel,
      text: cupsText(-1)
    })
    await addReactions({
      app: slack.app,
      channelId: event.channel,
      timestamp: sent.ts,
      reactions: ['one', 'two', 'three']
    })
    // const addReaction = async emojiName =>
    //   slack.app.client.reactions.add({
    //     channel: event.channel,
    //     timestamp: sent.ts,
    //     name: emojiName
    //   })
    // await addReaction('one')
    // await addReaction('two')
    // await addReaction('three')

    const ts = sent.message.ts
    const lastOne = {}
    const moves = 10
    for (let i = 0; i < moves; i++) {
      setTimeout(() => {
        slack.app.client.chat.update({
          channel: event.channel,
          ts,
          text: cupsText(lastOne.num = Math.floor(Math.random() * 3))
        })
      }, i * (200 - (i * 5)))
    }
    for (let i = moves + 1; i < moves + 4; i++) {
      setTimeout(() => {
        slack.app.client.chat.update({
          channel: event.channel,
          ts,
          text: cupsText(-1)
        })
      }, i * (200 - (i * 5)))
    }
    setTimeout(async () => {
      const timeoutSeconds = event.channel_type === 'im' ? 5 : 10
      for (let i = timeoutSeconds; i >= 0; i--) {
        setTimeout(async () => {
          await slack.app.client.chat.update({
            channel: event.channel,
            ts,
            text: cupsText(-1) + (i ? `\n ${i}` : '')
          })
        }, (1 + timeoutSeconds - i) * 1000)
      }
      setTimeout(async () => {
        const reactions = await slack.app.client.reactions.get({
          channel: event.channel,
          timestamp: ts,
          full: true
        })
        const reactPosters = {}
        reactions.message.reactions.forEach(r => r.users.forEach(user => {
          reactPosters[user] ??= []
          reactPosters[user].push(r.name)
        }))
        let winners = ``
        const nums = {
          one: 0,
          two: 1,
          three: 2,
        }
        Object.entries(reactPosters).forEach(([id, votes]) => {
          if (votes.length === 1 && nums[votes[0]] === lastOne.num) {
            winners += `<@${id}> `
          }
        })
        await slack.app.client.chat.update({
          channel: event.channel,
          ts,
          text: cupsText(lastOne.num)
        })
        await say(`It was hidden under Cup Number ${lastOne.num + 1}! Good guess, ${winners || `no one`}!`)
        delete activeCupsGames[event.channel]
      }, (timeoutSeconds + 1) * 1000)
    }, moves * (200 - (moves * 5)))
  }
)

command(
  ['!setpw'],
  'Set your api password. May not contain spaces. *This is not secure!*\n' +
    '  To use, say !setpw your_password',
  async ({ say, words, user }) => {
    if (words[2]) {
      return say(`Your password may not contain spaces!`)
    }
    user.pwHash = webapi.makeHash(words[1])
    await saveGame()
    await say(`Password encoded as ${user.pwHash}`)
  }
  , { hidden: true })

const getHvackerHelpCost = () => {
  horrors.hvackerHelp ??= 1
  return 1_000_000_000_000 * Math.pow(horrors.hvackerHelp * 3, 3)
}

command(
  ['!!help'],
  'Help someone in need.',
  async ({ say, words, event, user }) => {
    if (words[1] !== 'hvacker' && words[1] !== `<@${slack.users.Hvacker}>`) {
      return
    }
    
    if (!settings.horrorEnabled) {
      return say(`You know? I actually feel okay, thanks`)
    }

    setHighestCoins(event.user)
    const cost = getHvackerHelpCost()
    if (user.coins < cost) {
      return say(`You don't have enough coins to help right now.`)
    }
    user.coins -= cost
    horrors.hvackerHelp += 1
    horrors.hvackerLast = dayOfYear()
    saveGame()
    await say('I feel a bit better. Thank you...')
  }, { hidden: true })

command(
  ['!help', '!h'],
  'List available commands',
  async ({ say, words, user }) => {
    if (settings.horrorEnabled && (words[1] === 'hvacker' || words[1] === `<@${slack.users.Hvacker}>`)) {
      const cost = getHvackerHelpCost()
      const postfix = user.coins < cost ? `You don't have enough coins to help right now.` : `Say \`!!help hvacker\` to confirm.`
      await say(`_I need ${commas(cost)} Coins. Please..._\n${postfix}`)
      return
    }
    if (words[1] === 'full') {
      return say('```' + commandHelpText + '```')
    }
    return say(shortCommandHelpText + '```')
  }
)

const removeAchievement = async (user, name, say) => {
  if (user.achievements[name]) {
    user.achievements[name] = false
    await say('Achievement removed!')
  } else {
    await say('That user doesn\'t have that achievement!')
  }
}

command(
  ['!rach'],
  'Remove achievement',
  async ({ say, words }) => {
    const achName = words[1]
    const target = idFromWord(words[2])
    await removeAchievement(getUser(target), achName, say)
  }, adminOnly)

command(
  ['!a', '!ach', '!achievements'],
  'List your glorious achievements',
  async ({ event, say, user }) => {
    const achievementCount = Object.keys(user.achievements).length
    const prefix = `You have ${achievementCount} achievements!\n\n`
    const mult = (Math.pow(1.01, achievementCount) - 1) * 100
    const isIm = event.channel_type === 'im'
    const desc = isIm
      ? ({ description, emoji, name }) => `:${emoji}: *${name}* - ${description}`
      : ({ emoji }) => `:${emoji}:`

    const list = Object.keys(user.achievements)
        .map(name => achievements[name])
        .map(desc)
        .join(isIm ? '\n' : ' ') + '\n\n'

    const postfix = achievementCount ? `_Achievements are boosting your CPS by ${mult.toPrecision(3)}%_` : ''
    await say(prefix + list + postfix)
  }
)

const emojiLine2 = (itemName, countOwned) =>
  ((emoji = buyableItems[itemName].emoji) => countOwned < 5
    ? `:${emoji}:`.repeat(countOwned)
    : `:${emoji}: x${countOwned}`)()

const emojiLine3 = (itemName, countOwned) =>
  (countOwned < 5
    ? s => s.repeat(countOwned)
    : s => s + ` x${countOwned}`)(buyableItems[itemName].emoji)

const emojiLine = (itemName, countOwned) => countOwned < 5
  ? `:${buyableItems[itemName].emoji}:`.repeat(countOwned)
  : `:${buyableItems[itemName].emoji}: x${countOwned}`
const collection = user =>
  Object.entries(user?.items || {})
    .map(([itemName, countOwned]) => emojiLine(itemName, countOwned) + ' - ' + commas(getItemCps(user, itemName)) + ' cps')
    .join('\n')

command(['!cps'],
  'Display your current Coins Per Second',
  async ({ say, user }) =>
    say(`You are currently earning \`${commas(getCPS(user))}\` HVAC Coin per second.`))

// What's that one game? Split or Steal?

command(['!split'],
  `Play a game of split or steal.`
  , {hidden: true})


// const maxTemp = 30
// let miningHeat = 0
// const cool = () =>
//   setTimeout(() => {
//     if (miningHeat > 0) {
//       miningHeat -= 1
//     }
//     cool()
//   }, 1200)
// cool()
//
// const miningSites = {}
// slack.onReaction(async ({ event, say }) => {
//   if (event.reaction === 'pick') {
//     try {
//       if (!miningSites[event.item.ts]) {
//         console.log('New mining site!')
//         miningSites[event.item.ts] = true
//         await slack.app.client.reactions.add({
//           channel: event.item.channel,
//           timestamp: event.item.ts,
//           name: 'pick'
//         })
//       }
//     } catch (e) {}
//
//     const text = await doMine({ user: getUser(event.user), userId: event.user, say })
//     console.log('miningHeat:', miningHeat)
//     if (miningHeat < maxTemp) {
//       console.log(`${slack.users[event.user]} is pick mining.`);
//       miningHeat += 1
//       try {
//         await slack.app.client.chat.update({
//           channel: event.item.channel,
//           ts: event.item.ts,
//           text
//         })
//       } catch {}
//     } else if (miningSites[event.item.ts] !== 'hot') {
//       miningSites[event.item.ts] = 'hot'
//       try {
//         await slack.app.client.chat.update({
//           channel: event.item.channel,
//           ts: event.item.ts,
//           text: text + '\n\nThis mining site is getting too hot! I can\'t keep updating your CPS!'
//         })
//       } catch {}
//     }
//   }
// })

const doMine = async ({ user, userId, say }) => {
  const random = Math.random()
  const c = user.coins
  const secondsOfCps = seconds => Math.floor(getCPS(user) * seconds)
  let diff
  let prefix
  if (random > 0.9967) {
    diff = 500 + Math.floor(c * 0.10) + secondsOfCps(60 * 30)
    prefix = `:gem: You found a lucky gem worth ${commas(diff)} HVAC!\n`
    addAchievement(user, 'luckyGem', say)
    await slack.messageSage(`${slack.users[userId]} FOUND A LUCKY GEM COIN WORTH ${commas(diff)} HVAC!`)
  } else if (random > 0.986) {
    diff = 50 + Math.floor(c * 0.025) + secondsOfCps(60)
    prefix = `:goldbrick: You found a lucky gold coin worth ${commas(diff)} HVAC!\n`
    addAchievement(user, 'goldBrick', say)
  } else if (random > 0.96) {
    diff = 10 + Math.floor(c * 0.01) + secondsOfCps(10)
    prefix = `:money_with_wings: You found a lucky green coin worth ${commas(diff)} HVAC!\n`
    addAchievement(user, 'greenCoin', say)
  } else {
    prefix = 'You mined one HVAC.\n'
    diff = 1
  }
  user.coins = c + diff
  saveGame()
  return `${prefix}You now have ${commas(user.coins)} HVAC coin${c !== 1 ? 's' : ''}. Spend wisely.`
}

command(
  ['!c', '!coin', '!mine', '!'],
  'Mine HVAC coins',
  async ({ say, user, userId }) => {
    await say(await doMine({ user, userId, say }))
  }
)

command(
  ['!as'],
  'Run commands as another user.',
  async ({ event, words, trueSay }) => {
    const [bangAs, impersonating, ...newWords] = words
    event.user = idFromWord(impersonating)
    event.text = newWords.join(' ')
    await messageHandler({ event, say: trueSay, isRecycle: false})
  }, adminOnly)

command(
  ['!g', '!gamble'],
  'Gamble away your HVAC\n' +
    '  To use, say \'gamble coin_amount\' or \'!gamble all\'',
  async ({ event, say, words, user }) => {

    const [, ...wager] = words
    const requestedWager = parseAll(wager.join(' '), user.coins)
    const n = (chaosFilter(requestedWager, 0.2, user, user.coins) + requestedWager) / 2
    if (!n || n < 0) {
      return say(`Invalid number '${n}'`)
    }
    if (user.coins < n) {
      return say(`You don't have that many coins! You have ${commas(user.coins)}.`)
    }
    if (n >= 100_000_000_000) {
      addAchievement(user, 'bigBets', say)
    }
    user.coins -= n
    let outcome
    if (Math.random() > 0.5) {
      user.coins += (2 * n)
      outcome = 'won'
    } else {
      outcome = 'lost'
    }
    console.log(`They ${outcome}`)
    saveGame()
    await say(`You bet ${commas(n)} coins and ${outcome}! You now have ${commas(user.coins)}.`)
    if (outcome === 'lost') {
      switch (event.user) {
        case slack.users.Quade:
          await say(':trimedary_anim:')
          break;
        case slack.users.Houston:
          await say(':fortnite:')
          break;
      }
    }
  }
)

command(
  ['!buynft', '!bn'],
  'Acquire high-quality art\n' +
    '  To use, say \'!buynft nft_name\'',
  async ({ event, say, words, user }) => {
    const nft = nfts.find(n => n.name.toLowerCase() === words[1])
    if (!nft) {
      const suffix = words[1]?.match(/[^a-z0-9_]/i) ? '. And I\'m unhackable, so cut it out.' : ''
      return say('No NFT with that name found' + suffix)
    }
    if (nft.owner) {
      return say('Someone already owns that NFT!')
    }
    const c = user.coins
    if (c < nft.price) {
      return say('You don\'t have enough coin for this nft')
    }

    user.coins -= nft.price
    nft.owner = event.user
    saveGame()
    await say('You bought ' + nft.name + '!')
  }
)

command(
  ['!myupgrades', '!myu'],
  'List all the upgrades that you own.',
  async ({ say, user }) => {
    await say(upgradeText(user, true))
  }, dmsOnly)

command(
  ['!upgrade', '!u'],
  'Improve the performance of your HVAC-generators.\n' +
    '  Say \'!upgrade\' to list available upgrades, or \'!upgrade upgrade_name\' to purchase.',
  async ({ say, words, user }) => {
    const upgradeName = words[1]
    if (!upgradeName) {
      return say(upgradeText(user))
    }
    const upgrade = upgrades[upgradeName]
    if (!upgrade) {
      return say('An upgrade with that name does not exist!')
    }
    if (!user.upgrades[upgrade.type]) {
      user.upgrades[upgrade.type] = []
    }
    if (hasUpgrade(user, upgrade, upgradeName)) {
      return say('You already have that upgrade!')
    }
    if (!upgrade.condition(user)) {
      return say('That item does not exist!')
    }
    const c = user.coins
    if (c < upgrade.cost) {
      return say(`You don't have enough coins! You have ${commas(c)}, but you need ${commas(upgrade.cost)}`)
    }
    user.coins -= upgrade.cost
    user.upgrades[upgrade.type].push(upgradeName)
    await saveGame()
    await say(`You bought ${upgradeName}!`)
  }, dmsOnly)

const getCurrentSquadgrade = () => {
  const current = Object.entries(squadUpgrades).find(squadIsMissing)
  if (!current) {
    return current
  }
  const [name, upgrade] = current
  if (!squad.upgrades[name]) {
    squad.upgrades[name] = upgrade.cost
  }
  return {
    name,
    remaining: squad.upgrades[name],
    emoji: upgrade.emoji,
    description: upgrade.description
  }
}

const squadgradeText = ([name, { emoji, description }]) =>
    `:${emoji}: *${name}*\n_${description}_`

const squadText = () => {
  const current = getCurrentSquadgrade()
  if (current) {
    const currentUpgradeText = ({ name, remaining, emoji, description }) =>
        `:${emoji}: *${name}* - ${commas(remaining)} HVAC remaining.\n_${description}_`
    return currentUpgradeText(current)
  }
  return 'No more squadgrades currently available.'
  // const squadIsMissing = ([name, upgrade]) => squad.upgrades[name] === false
  // return squadgradeText(Object.entries(squadUpgrades).find(squadIsMissing))
}

command(
  ['!squad', '!sq'],
  'Buy upgrades that help the whole team.\n' +
    '  Say !squad to list squad upgrades.\n' +
    '  Say \'!squad contrib_amount\' to make progress toward the current upgrade.',
  async ({ say, words, user }) => {
    if (!words[1]) {
      return say(squadText())
    }
    const current = getCurrentSquadgrade()
    if (!current) {
      return say('No squadgrades are currently available')
    }
    const currentCoins = user.coins
    let amount = parseAll(words[1], currentCoins)
    if (amount > currentCoins) {
      return say(`You don't have that much HVAC! You have ${currentCoins}.`)
    }
    if (!amount || amount < 1) {
      return say(`Invalid amount: '${words[1]}'`)
    }
    if (amount > squad.upgrades[current.name]) {
      amount = squad.upgrades[current.name]
    }
    squad.upgrades[current.name] -= amount
    user.coins -= amount
    if (squad.upgrades[current.name] <= 0) {
      squad.upgrades[current.name] = true
    }
    saveGame()
    await say(`Thank you for your contribution of ${commas(amount)} HVAC! Current status:\n\n${squadText()}`)
  }
)

const buyRoute = require('./buy')
command(
  ['!buy', '!b'],
  'Buy new items to earn HVAC with\n' +
    '  Use without arguments to list all available items.\n' +
    '  Say \'!buy item_name optional_quantity\' to make your purchase.',
  buyRoute
)

command(
  ['!check', '!ch'],
  'Check how many coins another player has',
  async ({ say, words }) => {
    const targetId = idFromWord(words[1])
    if (!targetId) {
      return say('Target must be a valid @')
    }

    if (targetId === slack.users.Hvacker) {
      const members = (await slack.app.client.conversations.members({ channel: slack.temperatureChannelId })).members
      const humanMembers = members.filter(name => name.length === 11)
      return say(`Hvacker owns ${humanMembers.length} souls.`)
    }
    const coins = getCoins(targetId)
    await say(`<@${targetId}> has ${commas(coins, words[2] === 'exact')} HVAC.`)
  }
)

command(
  ['!gift', '!give', '!gi'],
  'Donate coins to a fellow player\n' +
    '  Send coins by saying \'!gift @player coin_amount\'',
  async ({ words, say, user }) => {
    const [, target, amountText] = words
    const amount = parseAll(amountText, user.coins)
    const targetId = idFromWord(target)
    if (!amount || amount < 0) {
      return say('Amount must be a positive integer!')
    }
    if (!targetId) {
      return say('Target must be a valid @')
    }
    if (user.coins < amount) {
      return say(`You don't have that many coins! You have ${commas(user.coins)} HVAC.`)
    }
    if (amountText === 'all' && slack.users.Tyler === targetId) {
      addAchievement(user, 'walmartGiftCard', say)
    }
    const targetUser = getUser(targetId)
    user.coins -= amount
    targetUser.coins += amount
    await say(`Gifted ${commas(amount)} HVAC to <@${targetId}>`)
  }
)

const getChaosMessage = (user, { channel_type }, prefix = '', postfix = '') => {
  const userQuackgrades = user.quackUpgrades?.cps || []
  if (userQuackgrades.includes('chaos') && channel_type?.endsWith('im')) {
    return `${prefix}Current chaos multiplier: ${getChaos(user)}${postfix}`
  }
  return ''
}

command(
  ['!status', '!s'],
  'Print your current CPS, HVAC balance, and owned items',
  async ({ event, say, user }) => {
    await say(
        `You are currently earning \`${commas(getCPS(user))}\` HVAC Coin per second.\n\n` +
        getChaosMessage(user, event, '', '\n\n') +
        `You currently have ${commas(user.coins)} HVAC Coins\n\n` +
        `${collection(user)}\n\nCoins collected all-time: ${commas(user.coinsAllTime)}\n\n`
    )
  }
)

command(
  ['!gimme'],
  'Give self x coins',
  async ({ say, words, user }) => {
    const increase = parseInt(words[1].replace(/,/g, ''))
    user.coins += increase
    user.coinsAllTime += increase
    await say(`You now have ${user.coins} HVAC.`)
  }, testOnly)

command(
  ['!nfts', '!nft', '!n'],
  'Show NFTs in the museum\n' +
    '  Call with no arguments to list all NFTs, or \'!nft nft_name\' to show a particular one.',
  async ({ say, words }) => {
    const owner = nft => `Owner: *${slack.users[nft.owner] || 'NONE'}*`
    const nftDisplay = nft => `_"${nft.name}"_\n\n${nft.description}\n\n${commas(nft.price)} HVAC.\n\n${nft.picture}\n\n${owner(nft)}`

    const filter = words[1] ? nft => words[1]?.toLowerCase() === nft.name : null

    await say(nfts
      .filter(filter || (() => true))
      .map(nftDisplay)
      .join('\n-------------------------\n') || (filter ? 'No NFTs with that name exist' : 'No NFTs currently exist.')
    )
  }
)

const prestigeEmoji = user => {
  const p = user.prestige || 0
  if (!p) {
    return ''
  }
  if (p < 10) {
    return ':star:'
  }
  if (p < 20) {
    return ':star2:'
  }
  if (p < 30) {
    return ':stars:'
  }
}

command(
  ['!leaderboard', '!lb'],
  'Show the top HVAC-earners, ranked by prestige, then CPS',
  async ({ say, user }) => {
    // if ((event.user === slack.users.Houston || event.user === slack.users.Sage) && event.channel_type.includes('im')) {
    //   return say('```' + `Hvacker - 9 souls - Taking from them whatever it desires\nSome other losers - who cares - whatever` + '```')
    // }
    let index = 1
    await say(
      Object.entries(users)
        .filter(([id, user]) => Object.entries(user.items).length > 0)
        .sort(([id, user1], [id2, user2]) => {
          const leftPrestige = getUser(id).prestige
          const rightPrestige = getUser(id2).prestige
          if (leftPrestige > rightPrestige) {
            return -1
          }
          if (rightPrestige > leftPrestige) {
            return 1
          }
          return getCPS(user1) > getCPS(user2)
        })
        .map(([id, u]) => `${index++}.  ${slack.users[id] || '???'} ${prestigeEmoji(u) || '-'} ${commas(getCPS(getUser(id)))} CPS - ${commas(getCoins(id))} HVAC`)
        .join('\n')
    ).then(() => addAchievement(user, 'leaderBoardViewer', say))
  }
)

const hints = []
const oneShot = (name, helpText, message, achievementName) => {
  if (helpText) {
    hints.push(helpText)
  }

  command([name], helpText, async ({ say, user }) => {
    await say(message)
    await slack.messageSage(`Wow buddy they like your ${name} joke.`)
    if (achievementName) {
      addAchievement(user, achievementName, say)
    }
  }, { hidden: true })
}

oneShot('!santa', 'Ho ho ho!', '<https://i.imgur.com/dBWgFfQ.png|I\'m Santa Quacks>')
oneShot('!sugma', 'Not very original.', ':hvacker_angery:')
oneShot('!pog', 'One poggers hvacker', '<https://i.imgur.com/XCg7WDz.png|poggers>')
oneShot('!ligma', 'Not very original.', '<https://i.imgur.com/i1YtW7m.png|no>')
oneShot('!dab', 'ACTIVATE COOL GUY MODE', '<https://i.imgur.com/FKYdeqo.jpg|I go XD style>', 'certifiedCoolGuy')
oneShot('!based', 'Sorry, it\'s a little hard to hear you!', '<https://i.imgur.com/IUX6R26.png|What?>')
oneShot('!shrek', 'Is love and is life.', '<https://i.imgur.com/QwuCQZA.png|Donkey!>')
// oneShot('!imaginedragons', 'The worst part about any group of white people is that they could be Imagine Dragons and you\'d have no way of knowing.', '<https://i.imgur.com/QwuCQZA.png|Donkey!>')

command(
  ['!hint'],
  '',
  async ({ say }) => say(`_${hints[Math.floor(Math.random() * hints.length)]}_`)
, {hidden: true})

command(
  ['!addnft'],
  'Arguments 1 and 2 should be on the first line as name (one word!) and integer price.\n' +
    '  The second line should be the description of the pieces\n' +
    '  the picture is everything after the second line',
  async ({ event }) => {
    const [, name, price] = event.text.substring(0, event.text.indexOf('\n')).split(' ')
    const rest = event.text.substring(event.text.indexOf('\n') + 1)
    const desc = rest.substring(0, rest.indexOf('\n'))
    const picture = rest.substring(rest.indexOf('\n') + 1)
    const newNft = {
      name,
      price: parseInt(price.replace(/,/g, '')),
      description: desc,
      picture,
      owner: null
    }
    nfts.push(newNft)
    console.log('addedNft', newNft)
    return saveGame()
  }, adminOnly)

command(
  ['!prestige', '!p'],
  'Show your current prestige status',
  prestige.prestigeRoute
)

command(
  ['!!prestige'],
  'Confirm your prestige activation.',
  prestige.prestigeConfirmRoute
)

command(
  ['!quack', '!quackstore'],
  'Spend your prestigious quackings\n\n' +
  'Say \'!quack upgrade_name\' to purchase a quackgrade',
  prestige.quackStoreRoute,
  prestigeOnly
)

command(
  ['!myquack', '!myq'],
  'See what kind of quackery you own.',
  prestige.ownedQuacksRoute,
  prestigeOnly
)

command(
  ['!lore', '!l'],
  'Sit down for a while. Maybe learn something.\n' +
  'You can use `!lore reset` to return to the beginning.',
  lore,
  dmsOnly
)

command(
  ['!ss'],
  'Show the status for another player: !ss @player',
  async ({ words, say }) => {
    const target = words[1]
    const targetId = idFromWord(target)
    const targetUser = getUser(targetId)
    await say(
          `${target} is currently earning \`${commas(getCPS(targetUser))}\` HVAC Coin per second.\n\n` +
          `They have ${commas(getCoins(targetId))} HVAC Coins\n\n` +
          `${collection(targetUser)}\n\n` +
          `${Object.entries(targetUser?.items || {}).reduce((total, [name, count]) => total + count, 0)} total items\n\n`
    )
  }, adminOnly)

const exec = require('child_process').exec
const fs = require('fs')
command(
  ['!pl'],
  '!pl `code`\n\n' +
  'A very very stupid lisp implementation.',
  async ({ event, say }) => {
    let code = ' '// = '(def iLoveHvacker "9jklFUlbnd38bCrrU9765FhN") '
    code += event.text.substring(4)
      .replace(/`/g, '')
      .replace(/&lt;/g, '<')
    code += ' '
    // console.log('PL CODE:', code)

    const fileName = '/home/sagevaillancourt/git/hvacker/pl/' + event.user + (new Date().toLocaleString().replace(/[^a-z0-9]/gi, '_'))
    fs.writeFileSync(fileName, code)

    const command = `/home/sagevaillancourt/projects/pebblisp/src/pl ${fileName}`
    console.log(command)
    const child = exec(command)
    let result = '```\n'
    let errors = ''
    child.stdout.on('data', data => (result += data.replace(/\[\d+m/g, '')))
    child.stderr.on('data', data => {
      result += `\nERR:${data}\n`
      errors += `\nERR:${data}`
    })
    const maxLines = 25
    const maxChar = 2000
    child.on('close', async () => {
      // fs.rmSync(fileName)
      const lines = result.split('\n')
      let err = '\n'
      if (lines.length > maxLines) {
        result = lines.slice(0, maxLines).join('\n') + '\n...'
        err += `Output exceeds ${maxLines} lines and was truncated.\n`
      }
      if (result.length > maxChar) {
        result = result.substring(0, maxChar)
        err += `\nOutput also exceeds ${maxChar} characters and was truncated further...\n`
      }
      result += '```' + err + (errors && 'stderr:\n```\n' + errors.substring(0, 200) + '```')
      await say(result)
    })
  }, { hidden: true, condition: ({ event }) => event.user.startsWith(slack.users.Sage) })

command(
  ['!steal', '!sagesteal'],
  'Highly illegal',
  async ({ event, words, say, user }) => {
    const [, target] = words
    const amount = user.coins / 100
    if (!amount || amount < 0) {
      return
    }
    let targetId = idFromWord(target)
    if (event.user === targetId) {
      return say('What, are you trying to steal from yourself? What, are you stupid?')
    }
    if (!targetId) {
      targetId = slack.users.Sage
    }
    if (user.coins < amount) {
      return
    }
    const targetUser = getUser(targetId)
    user.coins -= amount
    targetUser.coins += amount
    await say(`Stealing is wrong. Gave ${commas(amount)} of your HVAC to ${slack.users[targetId]}`)
  }, { hidden: true })

// TODO
command(
  ['!lotto'],
  'Once per day, try for a big win!',
  async ({ event, say, user }) => {
    const currentDate = new Date().getDate()
    const lastLotto = user.lastLotto === undefined ? currentDate - 1 : user.lastLotto
    if (lastLotto === currentDate && event.user !== slack.users.Sage) {
      return say('Hey, only one lotto ticket per day, alright?')
    }
    let msg
    if (Math.random() < 0.01) {
      const prize = user.coinsAllTime / 20
      msg = `Ayyyyy, we have a winner! You win ${commas(prize)} HVAC!`
      user.coins += prize
    } else {
      msg = `Sorry pal, today's not your lucky day.`
    }
    await say(msg)
    user.lastLotto = currentDate
    saveGame()
  }, { hidden: true })

command(
  ['!ignite'],
  'You found me!',
  async ({ say, user }) => {
    addAchievement(user, 'ignited', say)
  }, { hidden: true })

command(
  ['!giveach'],
  '!giveach @player ach_name',
  async ({ words, say, user }) => {
    addAchievement(user, words[2], say)
    saveGame()
  }, adminOnly)

command(
  ['!whois'],
  '!whois player_id',
  async ({ words, say }) => say(`<@${words[1]}>`),
  adminOnly)

command(
  ['!ngift'],
  '!ngift player_id nft_name',
  async ({ event, words, say }) => {
    const targetId = idFromWord(words[1])
    if (!targetId) {
      return say('Please specify a valid @ target!')
    }
    const nft = nfts.find(nft => nft.name === words[2])
    if (!nft) {
      return say(`There is not NFT named "${words[2]}"!`)
    }
    if (nft.owner !== event.user) {
      return say(`You do not own "${nft.name}"!`)
    }
    nft.owner = targetId
  }, { hidden: true })

command(
  ['!deletetest', '!dtest'],
  '!deletetest',
  async () => {
    delete users[slack.testId]
    saveGame()
  }, adminOnly)

command(
  ['!message', '!msg', '!m'],
  '!message player_id message',
  async ({ event, words, say }) => {
    const targetId = idFromWord(words[1])
    if (!targetId) {
      return say('Please specify a valid @ target!')
    }
    const msg = event.text.replace(/\S+ +\S+\s*/i, '')
    await slack.messageIn(targetId, msg)
  }, adminOnly)

// webapi.launch()
